package Irpg::Action;

use strict;
use warnings;
use POSIX;
use Irpg::Utils qw(:text);
use Irpg::Irc qw(:interaction);
use List::Util qw(min);
use Exporter qw(import);
our @EXPORT = qw(&choose_action &itemsum
                 &challenge_opp &collision_action &team_battle);
our @EXPORT_OK = qw(itemsum new_action);

my $primnick_ref;
my $opts;
my $rps;
=head1 FUNCTION init_pkg
    This function sets the references to
    options and players hashes.
=over
=item SCALAR (ref)    - reference to the options hash
=item SCALAR (ref)    - reference to the players hash
=cut
sub init_pkg {
    ($opts, $rps, $primnick_ref) = @_;
}


sub fisher_yates_shuffle {
    my $array = shift;
    my $i;
    for ($i = @$array; --$i; ) {
        my $j = int rand ($i+1);
        next if $i == $j;
        @$array[$i,$j] = @$array[$j,$i];
    }
}

sub itemsum {
    my $user = shift;
    # is this for a battle? if so, good users get a 10% boost and evil users get
    # a 10% detriment
    my $battle = shift;
    return -1 unless defined $user;
    my $sum = 0;
    if ($user eq $$primnick_ref) {
        for my $u (keys(%$rps)) {
            $sum = itemsum($u) if $sum < itemsum($u);
        }
        return $sum+1;
    }
    if (!exists($rps->{$user})) { return -1; }
    #$sum += int($rps->{$user}{item}{$_}) for keys(%{$rps->{$user}{item}});
    $sum += int foreach map { m/(\d+)\D?/ } values(%{$rps->{$user}{item}});
    if ($battle) {
        $sum = $rps->{$user}{alignment} eq 'Evil' ? int(0.5+$sum*.9) :
               $rps->{$user}{alignment} eq 'Good' ? int(0.5+$sum*1.1) :
               $sum;
        return int(0.5 + $rps->{$user}{class}->real_sum($sum));
    }
    return $sum;
}

sub choose_action {
    my $user = shift || return;
    my $phy_score = $rps->{$user}{class}->str() + $rps->{$user}{class}->con();
    my $mys_score = $rps->{$user}{class}->wis() + $rps->{$user}{class}->int();
    my $stl_score = $rps->{$user}{class}->cha() + $rps->{$user}{class}->dex();
    my $score = int(rand($phy_score+$mys_score+$stl_score));
    return 'fight' if ($score < $phy_score);
    return 'mystic' if ($score < $phy_score+$mys_score);
    return 'steal';
}

sub minimize {
    my $x = shift;
    return unless defined($x);
    return 5*exp(-pow($x,2)/5000);
}

sub maximize {
    my $x = shift;
    return unless defined($x);
    return 5+pow($x,2)/1000;
}

sub perc_defeat {
    my $lvl_gap = shift;
    return unless defined($lvl_gap);
    return 5 if $lvl_gap == 0;
    return $lvl_gap > 0 ? minimize($lvl_gap) : maximize($lvl_gap);
}

sub perc_victory {
    my $lvl_gap = shift;
    return unless defined($lvl_gap);
    return 5 if $lvl_gap == 0;
    return $lvl_gap > 0 ? maximize($lvl_gap) : minimize($lvl_gap);
}

sub fight_result {
    my ($p1, $p2, $win, $wanted) = @_;
    return unless $p1 && $p2 && defined($win);
    my @queue = ();
    if ($win) {
        #my $gain = int($rps->{$p2}{level}/4);
        #$gain = 7 if $gain < 7;
        my $gain = perc_victory($rps->{$p2}{level} - $rps->{$p1}{level});
        $gain *= $rps->{$p1}{class}->{MOD_CRT} if ($win == 2);
        $gain *= 1.2 if ($wanted);
        $gain = int(($gain/100)*$rps->{$p1}{next});
        $gain = -$rps->{$p1}{class}->real_gain(-$gain);
        $rps->{$p1}{next} = $gain >= $rps->{$p1}{next} ? 0 : $rps->{$p1}{next} - $gain;
        push(@queue,
            duration($gain)." is removed from $p1\'s clock.");
        if ($win == 2) {
            $gain = int(0.05*$rps->{$p2}{next});
            $rps->{$p2}{next} += $rps->{$p2}{class}->real_gain($gain);
            push(@queue,
                "$p1 has dealt $p2 a Critical Strike! ".
                duration($gain)." is added to $p2\'s clock. ".
                "$p2 reaches next level in\x035 ".duration($rps->{$p2}{next})."\x03.");
        }
    }
    else {
        #my $gain = $rps->{$p2}{level}/7;
        #$gain = 7 if $gain < 7;
        my $gain = perc_defeat($rps->{$p2}{level} - $rps->{$p1}{level});
        $gain *= 1.2 if ($wanted);
        $gain = int(($gain/100)*$rps->{$p1}{next});
        $gain = $rps->{$p1}{class}->real_gain($gain);
        $rps->{$p1}{next} += $gain;
        push(@queue,
            duration($gain)." is added to $p1\'s clock.");
    }
    $queue[0] .= " $p1 reaches next level in\x03".($win ? '3' : '5').
                 ' '.duration($rps->{$p1}{next})."\x03.";
    return \@queue;
}

sub mystic_result {
    my ($p1, $p2, $win, $wanted) = @_;
    return unless $p1 && $p2 && defined($win);
    my @queue = ();
    if ($win) {
        #my $gain = int($rps->{$p2}{level}/8);
        #$gain = 5 if $gain < 5;
        my $perc = perc_victory($rps->{$p2}{level} - $rps->{$p1}{level})*0.60;
        $perc *= 1.2 if ($wanted);
        my $gain_p1 = int(($perc/100)*$rps->{$p1}{next});
        my $gain_p2 = min($gain_p1, int(($perc/100)*$rps->{$p2}{next}));
        $gain_p1 = -$rps->{$p1}{class}->real_gain(-$gain_p1);
        $gain_p2 = $rps->{$p2}{class}->real_gain($gain_p2);
        $rps->{$p1}{next} = $gain_p1 >= $rps->{$p1}{next} ?
                            0 : $rps->{$p1}{next} - $gain_p1;
        $rps->{$p2}{next} += $gain_p2;
        push(@queue,
            "$p1 transmutes ".duration($gain_p1)." from ".pronoun(2, $rps->{$p1}{gender}).
            " clock to ".duration($gain_p2)." for $p2\'s. ".
            "$p1 reaches next level in\x033 ".duration($rps->{$p1}{next})."\x03. ".
            "$p2 reaches next level in\x035 ".duration($rps->{$p2}{next})."\x03.");
        if ($win == 2) {
            my $gain = int(0.05*$rps->{$p1}{next});
            $gain *= $rps->{$p1}{class}->{MOD_CRT};
            $gain = -$rps->{$p2}{class}->real_gain(-$gain);
            $rps->{$p1}{next} = $gain >= $rps->{$p1}{next} ? 0 : $rps->{$p1}{next} - $gain;
            push(@queue,
                "$p1 has cast a Critical Spell! ".
                duration($gain)." of ".pronoun(2, $rps->{$p1}{gender}).
                " clock is sent to another plane. ".
                "$p1 reaches next level in\x033 ".duration($rps->{$p1}{next})."\x03.");
        }
    }
    else {
        #my $gain = $rps->{$p2}{level}/10;
        #$gain = 5 if $gain < 5;
        my $perc = perc_defeat($rps->{$p2}{level} - $rps->{$p1}{level})*0.60;
        $perc *= 1.2 if ($wanted);
        my $gain_p2 = int(($perc/100)*$rps->{$p2}{next});
        my $gain_p1 = min($gain_p2, int(($perc/100)*$rps->{$p1}{next}));
        $gain_p1 = $rps->{$p1}{class}->real_gain($gain_p1);
        $gain_p2 = -$rps->{$p2}{class}->real_gain(-$gain_p2);
        $rps->{$p1}{next} += $gain_p1;
        $rps->{$p2}{next} = $gain_p2 >= $rps->{$p2}{next} ?
                            0 : $rps->{$p2}{next} - $gain_p2;
        push(@queue,
            "$p1 transmutes ".duration($gain_p2)." from ${p2}'s clock to ".
            duration($gain_p1)." on ".pronoun(2, $rps->{$p1}{gender})." own. ".
            "$p1 reaches next level in\x035 ".duration($rps->{$p1}{next})."\x03. ".
            "$p2 reaches next level in\x033 ".duration($rps->{$p2}{next})."\x03.");
    }
    return \@queue;
}

sub steal_result {
    my ($p1, $p2, $win, $wanted) = @_;
    return unless $p1 && $p2 && defined($win);
    my @queue = ();
    if ($win == 1) {
        my @items = ("ring","amulet","charm","weapon","helm","tunic",
                     "pair of gloves","set of leggings","shield",
                     "pair of boots");
        my $type = $items[rand(@items)];
        my ($item1,) = $rps->{$p1}{item}{$type} =~ m/(\d+)\D?/;
        my ($item2,) = $rps->{$p2}{item}{$type} =~ m/(\d+)\D?/;
        #if (int($rps->{$p2}{item}{$type}) > int($rps->{$p1}{item}{$type})) {
        if ($item2 > $item1) {
            my $tempitem = $rps->{$p1}{item}{$type};
            $rps->{$p1}{item}{$type}=$rps->{$p2}{item}{$type};
            $rps->{$p2}{item}{$type} = $tempitem;
            push(@queue,
                "$p1 leaves, smiling at the thought of ".
                pronoun(2, $rps->{$p1}{gender})." wicked deed, whilst $p2 ".
                "remains unaware of ".pronoun(2, $rps->{$p2}{gender}).
                " own demise.");
            Irpg::Irc::notice(
                "You (rightfully) acquired a new level $item2 $type, which ".
                "looks far better than your old level $item1 $type! ".
                "You left your old one to whom you took the new from.",
                $rps->{$p1}{nick});
            Irpg::Irc::notice(
                "Waking up from your daydreaming, you realize your level ".
                "$item2 $type has been stolen, and replaced by a level ".
                "$item1 $type!",
                $rps->{$p2}{nick});
        }
        else {
            push(@queue,
                "Unfortunately, $p2\'s stolen \x035equipement revealed ".
                "itself to be useless\x03 to $p1, who returns it ".
                "to its rightful owner.");
        }
    }
    elsif ($win == 2) {
        push(@queue, "$p1 has managed a Critical Theft! ");
        my $type = "ring";
        my $val = 0;
        while( my ($k,$v) = each(%{$rps->{$p2}{item}}) ) {
            my ($v_d,) = $v =~ m/(\d+)\D?/;
            ($type, $val) = ($k, $v_d) if ($v_d > $val)
        }
        my ($item1,) = $rps->{$p1}{item}{$type} =~ m/(\d+)\D?/;
        my ($item2,) = $rps->{$p2}{item}{$type} =~ m/(\d+)\D?/;
        if ($val > $item1) {
            my $tempitem = $rps->{$p1}{item}{$type};
            $rps->{$p1}{item}{$type} = $rps->{$p2}{item}{$type};
            $rps->{$p2}{item}{$type} = $tempitem;
            $queue[$#queue] .= ucfirst(pronoun(1, $rps->{$p1}{gender}))." has retrieved ".
                "the finest of $p2\'s equipment. $p1 then proceeds to crawl ".
                "back into the shadows ".pronoun(1, $rps->{$p1}{gender})." came ".
                "from, ".pronoun(2, $rps->{$p1}{gender})." nefarious deed ".
                "unknown from any soul but ".pronoun(3, $rps->{$p1}{gender}).
                "self.";
            Irpg::Irc::notice(
                "You (rightfully) acquired a new level $item2 $type, which ".
                "looks far better than your old level $item1 $type! ".
                "You left your old one to whom you took the new from.",
                $rps->{$p1}{nick});
            Irpg::Irc::notice(
                "Waking up from your daydreaming, you realize your level ".
                "$item2 $type has been stolen, and replaced by a level ".
                "$item1 $type!",
                $rps->{$p2}{nick});
        }
        else {
            $queue[$#queue] .= "Alas, despite ".pronoun(2, $rps->{$p1}{gender}).
                " unquestionnable efforts, $p2\'s stolen equipement ".
                "revealed itself to be useless to $p1, who returns it ".
                "to its rightful owner.";
        }

    }
    else {
        my @items = ("ring","amulet","charm","weapon","helm","tunic",
                     "pair of gloves","set of leggings","shield",
                     "pair of boots");
        my $type = $items[rand(@items)];
        my ($val, $suffix) = ($rps->{$p1}{item}{$type} =~ /(\d+)(\D?)$/);
        $rps->{$p1}{item}{$type} = int(($val * .95)+0.5);
        $rps->{$p1}{item}{$type} .= $suffix;
        push(@queue,
            "$p1 did a false move, and damaged ".
            pronoun(2, $rps->{$p1}{gender})." $type, which lost ".
            "5% of its efficiency.");
    }
    return \@queue;
}

sub perform_action {
    my ($p1, $p2, $action_type) = @_;
    return unless $p1 && $p2 && $action_type;
    my $p1atk = eval '$rps->{$p1}{class}->'.$action_type.'_atk()';
    my $p2def = eval '$rps->{$p2}{class}->'.$action_type.'_def()';
    my $p1sum = itemsum($p1,1);
    my $p2sum = itemsum($p2,1);
    my $p1roll = int(rand($p1sum));# * $p1atk);
    my $p2roll = int(rand($p2sum));# * $p2def);
    my $ret;
    $ret = {p1sum=>$p1sum, p2sum=>$p2sum,
            p1roll=>$p1roll, p2roll=>$p2roll,
            p1atk=>$p1atk, p2def=>$p2def,
            vict=>0};
    if ($p1roll*$p1atk >= $p2roll*$p2def) {
        # VICTORY
        $ret->{vict}++;
        my $align_mod = $rps->{$p1}{alignment} eq 'Good' ? 0.05 :
                        $rps->{$p1}{alignment} eq 'Evil' ? -0.05 :
                        0;
        if ($p1roll*$p1atk >= $p1sum*$p1atk*($rps->{$p1}{class}->{CRIT_SK} + $align_mod)) {
            # CRITICAL STRIKE
            $ret->{vict}++;
        }
    }
    return $ret;
}

sub challenge_opp { # pit argument player against random player
    my $p1 = shift;
    if ($rps->{$p1}{level} < 25) { return unless rand(4) < 1; }
    my @opps = grep { $rps->{$_}{online} && $p1 ne $_ } keys(%$rps);
    return unless @opps;
    my $p2 = $opps[int(rand(@opps))];
    my $action_type = choose_action($p1);
    my $ret_action = perform_action($p1, $p2, $action_type);
    my $res_action = eval $action_type.'_result($p1, $p2, $ret_action->{vict})';
    my $p1_res = "[$ret_action->{p1roll}/$ret_action->{p1sum}]*$ret_action->{p1atk}";
    my $p2_res = "[$ret_action->{p2roll}/$ret_action->{p2sum}]*$ret_action->{p2def}";
    my $mesg = '';

    if ($action_type eq 'fight') {
        $mesg .=    "$p1 $p1_res has challenged $p2 $p2_res in combat and \x03".
                ($ret_action->{vict} ? "3won" : "5lost")."!";
    }
    elsif ($action_type eq 'mystic') {
        $mesg .= "$p1 $p1_res has cast a spell on $p2 $p2_res and \x03".
                ($ret_action->{vict} ? "3succeed" : "5failed")."!";
    }
    else { #($action_type eq 'steal')
        $mesg .= "$p1 $p1_res has tried to steal from $p2 $p2_res and \x03".
                ($ret_action->{vict} ? "3succeed" : "5failed")."!";
    }
    $mesg .= "\x03 ";
    $mesg .= shift @$res_action;
    Irpg::Irc::chanmsg(Irpg::Utils::clog($mesg));
    foreach (@$res_action) { Irpg::Irc::chanmsg(Irpg::Utils::clog($_)); }
}


sub collision_action {
    my($p1,$p2) = @_;
    my $action_type = choose_action($p1);
    my $ret_action = perform_action($p1, $p2, $action_type);
    my $res_action = eval $action_type.'_result($p1, $p2, $ret_action->{vict})';

    my $p1_res = "[$ret_action->{p1roll}/$ret_action->{p1sum}]*$ret_action->{p1atk}";
    my $p2_res = "[$ret_action->{p2roll}/$ret_action->{p2sum}]*$ret_action->{p2def}";
    my $mesg = "$p1 $p1_res has come upon $p2 $p2_res and ";
    $mesg .= $ret_action->{vict} ? "\x033" : "\x035";
    if ($action_type eq 'fight') {
        $mesg .= ($ret_action->{vict} ?
                    'taken '.pronoun(3, $rps->{$p2}{gender}):
                    'been defeated').
                " in combat!";
    }
    elsif ($action_type eq 'mystic') {
        $mesg .= ($ret_action->{vict} ?
                    'made '.pronoun(3, $rps->{$p2}{gender}).
                        ' lost '.pronoun(2, $rps->{$p2}{gender}):
                    'lost '.pronoun(2, $rps->{$p1}{gender})).
                " minds!";
    }
    else { #($action_type eq 'steal')
        $mesg .= ($ret_action->{vict} ?
                    'stolen ':
                    'been caught stealing ').
                'from '.pronoun(3, $rps->{$p2}{gender})."!";
    }
    $mesg .= "\x03 ";
    $mesg .= shift @$res_action;
    Irpg::Irc::chanmsg(Irpg::Utils::clog($mesg));
    foreach (@$res_action) { Irpg::Irc::chanmsg(Irpg::Utils::clog($_)); }
}

sub team_battle { # pit three players against three other players
    my @opp = grep { $rps->{$_}{online} } keys(%$rps);
    return if @opp < 6;
    splice(@opp,int(rand(@opp)),1) while @opp > 6;
    fisher_yates_shuffle(\@opp);

    # the gain is 20% of the lower TTL of defenders
    my $gain = $rps->{$opp[0]}{next};
    for my $p (1,2) {
        $gain = $rps->{$opp[$p]}{next} if $gain > $rps->{$opp[$p]}{next};
    }
    $gain = int($gain*.20);

    # the fight, every one picks an action, and roll with its modificator
    my ($mysum, $oppsum) = (0, 0);
    my ($myroll, $opproll) = (0, 0);
    my ($action_type, $sum, $mod);
    for (my $i = 0; $i < 6 ; $i++) {
        $action_type = choose_action($opp[$i]);
        $sum = int(itemsum($opp[$i],1));
        if ($i < 3) {
            $mysum += $sum;
            $mod = eval '$rps->{$opp[$i]}{class}->'.$action_type.'_atk()';
            $myroll += int(rand($sum) * $mod);
        }
        else {
            $oppsum += $sum;
            $mod = eval '$rps->{$opp[$i]}{class}->'.$action_type.'_def()';
            $opproll += int(rand($sum) * $mod);
        }
    }

    my ($state, $moved);
    if ($myroll >= $opproll) {
        ($state, $moved) = ('3won', 'removed from');
        for (my $i = 0; $i < 3; $i++) {
            $rps->{$opp[$i]}{next} += $rps->{$opp[$i]}{class}->real_gain(-$gain);
        }
    }
    else {
        ($state, $moved) = ('5lost', 'added to');
        for (my $i = 0; $i < 3; $i++) {
            $rps->{$opp[$i]}{next} += $rps->{$opp[$i]}{class}->real_gain($gain);
        }
    }
    Irpg::Irc::chanmsg(Irpg::Utils::clog(
        "$opp[0], $opp[1], and $opp[2] ".
        "[".($myroll/5)."/".($mysum/5)."] have team battled ".
        "$opp[3], $opp[4], and $opp[5] ".
        "[".($opproll/5)."/".($oppsum/5)."] and \x03$state! ".
        duration($gain)."\x03 is $moved their clocks."));
}

sub rpcheck {
    my $rpreport = shift;
    if ($rpreport%3600==0 && $rpreport) { # 1 hour
        my @players = grep { $rps->{$_}{online} &&
                             $rps->{$_}{level} > 44 } keys(%$rps);
        # 20% of all players must be level 45+
        if ((scalar(@players)/scalar(grep { $rps->{$_}{online} } keys(%$rps))) > .15) {
            challenge_opp($players[int(rand(@players))]);
        }
    }
}

sub do_actions {
    my ($userhost, $usernick, $username, $source, @arg) = @_;
    my $ret;
    do {
        $ret = do_action($userhost, $usernick, $username, $source, @arg);
    } while ($rps->{$username}{actions} && !$ret);
}

sub do_action {
    my ($userhost, $usernick, $username, $source, @arg) = @_;
    if (!defined($username)) {
        Irpg::Irc::notice("You are not logged in.", $usernick);
        return 1;
    }
    if (!$rps->{$username}{actions}) {
        Irpg::Irc::notice("You feel too weak to do anything.", $usernick);
        Irpg::Irc::chanmsg("$username seems to be trying to achieve ".
            "an undefined deed, but collapses out of exhaustion.");
        return 1;
    }
    my ($type,) = join(' ', @arg) =~ m/-t (\w+)/;
    if (defined($type)) {
        if (!($type =~ m/fight|mystic|steal/i)) {
            Irpg::Irc::notice("Despite your intensive training, ".
                "you are still unable to perform a '$type' action.", $usernick);
            return 1;
        }
        #remove "-t xxx" from @arg
        foreach ('-t', $type) {
            my $index = 0;
            $index++ until $arg[$index] eq $_;
            splice(@arg, $index, 1);
        }
    }
    if ($rps->{$username}{actions} < 6 && exists($arg[0]) && defined($type)) {
        Irpg::Irc::notice("You are far to weak willed and feeble to ".
            "both choose for yourself the action you want to perform and ".
            "look for anyone.", $usernick);
        Irpg::Irc::chanmsg("$usernick fails to handle his own path as the ".
            "Gods did not made ".pronoun(3, $rps->{$username}{gender})." ".
            "for such inanity.");
        return 1;
    }
    if ($rps->{$username}{actions} < 4 && defined($type)) {
        Irpg::Irc::notice("You do not have enough initiative to ".
            "choose for yourself the action you want to perform.",
            $usernick);
        Irpg::Irc::chanmsg("$username tries to overstep what the Gods ".
            "have planned for ".pronoun(3, $rps->{$username}{gender}).
            ", but fails.");
        return 1;
    }
    if ($rps->{$username}{actions} < 2 && exists($arg[0])) {
        Irpg::Irc::notice("You do not have enough energy to ".
            "look for anyone.", $usernick);
        Irpg::Irc::chanmsg("$username tries to contact someone, but ".
            pronoun(2, $rps->{$username}{gender})." call dies with ".
            pronoun(2, $rps->{$username}{gender})." breath.");
        return 1;
    }
    my @opps = grep { $rps->{$_}{online} && $username ne $_ } keys(%$rps);
    if (!@opps) {
        Irpg::Irc::notice("Your call echoes in the ambient silence...",$usernick);
        Irpg::Irc::chanmsg(
            "$username is desperatly searching for someone, ".
            "but ".pronoun(2, $rps->{$username}{gender})." call remains ".
            "unheard in the silent desert of ".pronoun(2, $rps->{$username}{gender}).
            " loneliness.");
        return 1;
    }
    my $p1 = $username;
    my $p2;
    if (exists($arg[0]) && !(grep { $arg[0] eq $_ } @opps)) {
        if (exists($rps->{$arg[0]})) {
            Irpg::Irc::notice(
                "You searched every places you know, but $arg[0] ".
                "is nowhere to be found...", $usernick);
        }
        else {
            Irpg::Irc::notice(
                "You asked everyone you met, but no one ever ".
                "heard of $arg[0].", $usernick);
        }
        Irpg::Irc::chanmsg(
            "$username calls $arg[0] for an honest interaction, but no one ".
            "can answer ".pronoun(2, $rps->{$username}{gender})." call.");
        return 1;
    }
    elsif (exists($arg[0])) {
        $p2 = $arg[0];
        # choosing your adversary cost an extra action point
        consume_action($username);
    }
    else {
        $p2 = $opps[int(rand(@opps))];
    }
    my $action_type = choose_action($p1);;
    if (defined($type)) {
        $action_type = $type;
        # selecting an action's type cost 3 extra action points
        consume_action($username);
        consume_action($username);
        consume_action($username);
    }
    consume_action($username);
    my $ret_action = perform_action($p1, $p2, $action_type);
    my $res_action = eval $action_type.'_result($p1, $p2, $ret_action->{vict}, 1)';

    my $p1_res = "[$ret_action->{p1roll}/$ret_action->{p1sum}]*$ret_action->{p1atk}";
    my $p2_res = "[$ret_action->{p2roll}/$ret_action->{p2sum}]*$ret_action->{p2def}";
    my $mesg = '';

    if ($action_type eq 'fight') {
        $mesg .= "$p1 $p1_res has provoked $p2 $p2_res in a fight, and \x03".
                ($ret_action->{vict} ? "3won": "5lost")."!";
    }
    elsif ($action_type eq 'mystic') {
        $mesg .= "$p1 $p1_res has performed mystical deeds on $p2 $p2_res, and \x03".
                ($ret_action->{vict} ?
                    "3dazed ".pronoun(3, $rps->{$p2}{gender}):
                    "5got confused").
                "!";
    }
    else { #($action_type eq 'steal')
        $mesg .= "$p1 $p1_res has sneaked on $p2 $p2_res in the shadows and \x03".
                ($ret_action->{vict} ? "3stolen " : "5been caught stealing ").
                'from '.pronoun(3, $rps->{$p2}{gender})."!";
    }
    $mesg .= "\x03 ";
    $mesg .= shift @$res_action;
    Irpg::Irc::chanmsg(Irpg::Utils::clog($mesg));
    foreach (@$res_action) { Irpg::Irc::chanmsg(Irpg::Utils::clog($_)); }
}

sub consume_action {
    my $player = shift;
    return unless (exists($rps->{$player}));
    $rps->{$player}{actions}--;
    push $rps->{$player}{next_a}, 3600;
}

sub new_action {
    my $player = shift;
    return unless (exists($rps->{$player}));
    $rps->{$player}{actions}++;
    Irpg::Irc::notice("You feel ready to perform a new deed !",
                      $rps->{$player}{nick});
}

our $commands = {
    action => {ref => \&do_action, adm => 0, prv => 1, pub => 1,
               hlp => 'ACTION [-t <type>] [<char name>]: perform an action '.
                      '(fight/mystic/steal) with someone. With no type specified, '.
                      'The action is chosen at random, '.
                      'with more chance for the one you are the better at.'},
    actions => {ref => \&do_actions, adm => 0, prv => 1, pub => 1,
                hlp => 'ACTIONS [-t <type>] [<char name>]: perform all available actions '.
                       'with someone. With no type spcified, the actions are chosen '.
                       'at random, with more chance for the one you are the better at.'}
};

1;
