package Irpg::Classes::Farmer;

use strict;
use warnings;

sub new {
	my ($class, $pl_stats) = @_;
	my $self = {};
	bless $self, $class;
	$self->{STATS} = $pl_stats;
	$self->{NAME} = 'Farmer';

	#modificators
	$self->{MOD_STR} = 1; # strength
	$self->{MOD_CON} = 1; # constitution
	$self->{MOD_WIS} = 1; # wisdom
	$self->{MOD_INT} = 1; # intelligence
	$self->{MOD_CHA} = 1; # charisma
	$self->{MOD_DEX} = 1; # dexterity
	$self->{CRIT_SK} = 0.90; # critical strike minimum range, the lower the largest acceptance
	$self->{MOD_CRT} = 1; # criticak strike modificator
	return $self;
}

sub str {
	my $self = shift;
	return $self->{STATS}{str} * $self->{MOD_STR};
}

sub con {
	my $self = shift;
	return $self->{STATS}{con} * $self->{MOD_CON};
}

sub wis {
	my $self = shift;
	return $self->{STATS}{wis} * $self->{MOD_WIS};
}

sub int {
	my $self = shift;
	return $self->{STATS}{int} * $self->{MOD_INT};
}

sub cha {
	my $self = shift;
	return $self->{STATS}{cha} * $self->{MOD_CHA};
}

sub dex {
	my $self = shift;
	return $self->{STATS}{dex} * $self->{MOD_DEX};
}

sub fight_atk {
	my $self = shift;
	return ($self->str() + $self->dex())/2;
}
sub fight_def {
	my $self = shift;
	return ($self->con() + $self->dex())/2;
}

sub mystic_atk {
	my $self = shift;
	return ($self->int() + $self->con())/2;
}
sub mystic_def {
	my $self = shift;
	return ($self->wis() + $self->con())/2;
}

sub steal_atk {
	my $self = shift;
	return ($self->dex() + $self->int())/2;
}
sub steal_def {
	my $self = shift;
	return ($self->cha() + $self->int())/2;
}

sub real_gain {
	# to apply on TTL modificators
	my ($self, $time) = @_;
	return unless ($time =~ m/^-?\d+(?:\.\d+)?$/);
	$time = $time >= 0 ?
		$time*(1-($self->cha()-1)/50):	# add less time
		$time*(1+($self->cha()-1)/50);	# remove more time
	return CORE::int($time);
}

sub real_sum {
	# to apply on itemsum
	my ($self, $sum) = @_;
	return unless ($sum =~ m/^-?\d+$/);
	$sum = $sum >= 0 ?
		$sum*(1+($self->str()-1)/50):
		$sum*(1-($self->str()-1)/50);
	return CORE::int($sum);
}

sub real_lvl {
	# to apply on found items
	my ($self, $lvl) = @_;
	return unless ($lvl =~ m/^-?\d+$/);
	$lvl = $lvl >= 0 ?
		$lvl*(1+($self->wis()-1)/50):
		$lvl*(1-($self->wis()-1)/50);
	return CORE::int($lvl);
}

1;
