#!/usr/bin/perl
# irpg bot v4.0.0 by ElTata, eltata@firemail.cc, See https://gennuso.iiens.net/irpg/
# Based on the  
# irpg bot v3.1.2 by jotun, jotun@idlerpg.net, et al. See http://idlerpg.net/
#
# Some code within this file was written by authors other than myself. As such,
# distributing this code or distributing modified versions of this code is
# strictly prohibited without written authorization from the authors. Contact
# jotun@idlerpg.net. Please note that this may change (at any time, no less) if
# authorization for distribution is given by patch submitters.
#
# As a side note, patches submitted for this project are automatically taken to
# be freely distributable and modifiable for any use, public or private, though
# I make no claim to ownership; original copyrights will be retained.. except as
# I've just stated.
#
# Please mail bugs, etc. to me. Patches are welcome to fix bugs or clean up
# the code, but please do not use a radically different coding style. Thanks
# to everyone that's contributed!
#
# NOTE: This code should NOT be run as root. You deserve anything that happens
#       to you if you run this code as a superuser. Also, note that giving a
#       user admin access to the bot effectively gives them full access to the
#       user under which your bot runs, as they can use the PEVAL command to
#       execute any command, or possibly even change your password. I sincerely
#       suggest that you exercise extreme caution when giving someone admin
#       access to your bot, or that you disable the PEVAL command for non-owner
#       accounts in your config file, .irpg.conf

use strict;
use warnings;
use Getopt::Long;
use Data::Dumper;

use Irpg::Irc qw(:connection &privmsg);
use Irpg::Utils qw(:DEFAULT &daemonize &set_debug_status :data);
use Irpg::Main;

my $opts = readconfig();

my $version = "5.2.1";

sub help { # print help message
    (my $prog = $0) =~ s/^.*\///;

    print "
usage: $prog [OPTIONS]
  --help, -h           Print this message
  --verbose, -v        Print verbose messages
  --server, -s         Specify IRC server:port to connect to
  --botnick, -n        Bot's IRC nick
  --botuser, -u        Bot's username
  --botrlnm, -r        Bot's real name
  --botchan, -c        IRC channel to join
  --listenchan         Other channels you want the bot to listen to
  --botident, -p       Specify identify-to-services command
  --botmodes, -m       Specify usermodes for the bot to set upon connect
  --botopcmd, -o       Specify command to send to server on successful connect
  --botghostcmd, -g    Specify command to send to server to regain primary
                       nickname when in use
  --doban              Advertisement ban on/off flag
  --debug              Debug on/off flag
  --helpurl            URL to refer new users to
  --admincommurl       URL to refer admins to

  Timing parameters:
  --rpbase             Base time to level up
  --rpstep             Time to next level = rpbase * (rpstep ** CURRENT_LEVEL)
  --rppenstep          PENALTY_SECS=(PENALTY*(RPPENSTEP**CURRENT_LEVEL))

";
}

# command line overrides .irpg.conf
GetOptions($opts,
    "help|h",
    "verbose|v",
    "debug",
    "debugfile=s",
    "server|s=s",
    "botnick|n=s",
    "botuser|u=s",
    "botrlnm|r=s",
    "botchan|c=s",
    "listenchan=s@",
    "botident|p=s",
    "botmodes|m=s",
    "botopcmd|o=s",
    "localaddr=s",
    "botghostcmd|g=s",
    "helpurl=s",
    "playerurl=s",
    "admincommurl=s",
    "doban",
    "silentmode=i",
    "writequestfile",
    "questfilename=s",
    "voiceonlogin",
    "noccodes",
    "nononp",
    "mapurl=s",
    "statuscmd",
    "pidfile=s",
    "reconnect",
    "reconnect_wait=i",
    "self_clock=i",
    "modsfile=s",
    "casematters",
    "detectsplits",
    "splitwait=i",
    "allowuserinfo",
    "noscale",
    "phonehome",
    "owner=s",
    "owneraddonly",
    "ownerdelonly",
    "ownerpevalonly",
    "checkupdates",
    "senduserlist",
    "limitpen=i",
    "mapx=i",
    "mapy=i",
    "modesperline=i",
    "okurl|k=s@",
    "eventsfile=s",
    "rpstep=f",
    "rpbase=i",
    "rppenstep=f",
    "dbfile|irpgdb|db|d=s",
) or debug("Error: Could not parse command line. Try $0 --help\n",1);

$opts->{help} and do { help(); exit 0; };

debug("Config: read $_: ".Dumper($opts->{$_})) for keys(%$opts);

my $rps = {}; # role-players
my $prev_online = {}; # user@hosts online on restart, die
my $auto_login = {}; # users to automatically log back on
my $lasttime = 1; # last time that rpcheck() was run

$rps = createdb($opts) unless -e $opts->{dbfile};

#print "\n".debug("Becoming a daemon...")."\n";
#daemonize();

$SIG{HUP} = "readconfig"; # sighup = reread config file

CONNECT:

$prev_online = loaddb($rps, 1);

irc_connect($opts);

Irpg::Main::init_pkg($opts, $rps, \$lasttime, $prev_online);

#-----------------#
#    MAIN LOOP    #
#-----------------#
while (1) {
    my @lines = checkmsg();
    if ($lines[0]) {
        parse($_) foreach (@lines[1..$#lines]);
    }
    else {
        # uh oh, we've been disconnected from the server, possibly before
        # we've logged in the users in %auto_login. so, we'll set those
        # users' online flags to 1, rewrite db, and attempt to reconnect
        # (if that's wanted of us)
        $rps->{$_}{online}=1 for keys(%$auto_login);
        writedb($rps);
        if ($opts->{reconnect}) {
            irc_clean($opts);
            goto CONNECT;
        }
        else { debug("Socket closed; disconnected.",1); }
    }
    if ((time()-$lasttime) >= $opts->{self_clock}) { rpcheck(); }
}
